# By AWARE on RC: python3.6 and it happy this *local* version of dataclasses, but ver>3.6 should go with system-wide version
try:
    from dataclasses import dataclass
except:
    from dataclasses_local import dataclass

from pathlib     import Path
from xml.dom.minidom import Element
from .remote_interfaces import RemoterInterface
import subprocess, shlex
import xml.etree.ElementTree as ET

# Requires GSUTIL
# https://cloud.google.com/storage/docs/gsutil_install#deb
# Might be a good option: snap install google-cloud-cli --classic
# After this you should initialize: gcloud init

class FireBaseException(Exception):
    pass

@dataclass
class Firebase(RemoterInterface):
    internal_name: str  = "[Firebase]"
    debug        : bool = False
    verbose      : bool = False
    #
    build_type       : str  = None  # debug/release
    release_shortcut : str  = None
    state            : str  = None  # deploy_type

    def __post_init__(self):

        if self.build_type       is None: FireBaseException("You have to define `build_type` ")
        if self.release_shortcut is None: FireBaseException("You have to define `release_shortcut` ")
        if self.state            is None: FireBaseException("You have to define `state` ")

    # used an interface by gutils
    def connect(self):
        pass
    
    def is_connected(self) -> bool:
        True
    
    def disconnect(self):
        pass        
    
    def upload(self, src_fname: Path = None, dst_fname: Path = None):
        cmd = f"gsutil cp -r {str(src_fname)} {dst_fname}"
        
        if self.debug: print(f"{self.internal_name} Uploading... [{cmd}]")
        
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"{output.stderr}")
        # typically empty
        # if output.stdout:
        #     print(f"Uploading output: {output.stdout}")

    def download(self, src_fname: Path = None, dst_fname: Path = None):
        cmd = f"gsutil cp -r {src_fname} {str(dst_fname)}"
        
        if self.debug: print(f"{self.internal_name} Downloading... [{cmd}]")

        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"{output.stderr}")

    def exists(self, path: Path) -> bool:
        cmd = f"gsutil ls gs://boson-{self.build_type}.appspot.com/Releases-{self.release_shortcut}/{self.state}/{path.name}"
        
        if self.debug: print(f"{self.internal_name} Checking... [{cmd}]")
        
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")

        for o in output.stdout.split():
            if self.debug: print(f"Checking {o}")
            if path.name == Path(o).name:
                print("I found something")
                return True
        return False
    
    '''
    Creating folder is not good option because of Google representation file...
    see: https://stackoverflow.com/questions/51892343/google-gsutil-create-folder
    '''
    def execute(self, cmd):
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"Execute output: {output.stderr}")
    

    def read_xml(self) -> Element:
        cmd = f"gsutil cat gs://boson-{self.build_type}.appspot.com/Releases-{self.release_shortcut}/{self.state}/index.xml"

        if self.debug: print(f"{self.internal_name} Checking... [{cmd}]")
        
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")

        try:
            root = ET.fromstring(output.stdout)
        except Exception as e:
            print("Problem with reading: {e}")
        
        return root